unit Diagram;

{
   The following is copyrighted material.

   (c) 1993, 1995 by Michael Leahy
                     BOOKUP Corp.
                     2763 Kensington Place West
                     Columbus, OH  43202-2355
                     U.S.A.

   (800) 949-5445  USA and Canada
   (614) 263-7219  International
   (614) 262-9788  Fax


   Version 1.0b Sep 21, 1995.
}

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs,

  DiagType,
  Position;

type
  TVersionString = String [5];

  TChessDiagram = class(TCustomControl)
{  private}
public
    { Private declarations }

{  protected}
    { Protected declarations }
    fVersion : TVersionString;
    fWhiteIsAtBottom : Boolean;
    fItsChessMove : ChessMoveType;
    fChessPosition : TChessPosition;
    fPromotionPiece : PieceType;
    fDraggingAPiece : Boolean;
    fAnimatingAMove : Boolean;
    fPieceBeingDragged : PieceType;
    fOriginalSquare,
    fTargetSquare : SquareType;

    procedure RefreshSquare (theSquare : SquareType); virtual;
    procedure DrawPiece (PieceNumber : PieceType; Hor, Ver : Integer); virtual;
    procedure ConstrictCoordinates (var X, Y : Integer); virtual;
    procedure StartAnimation (PieceNumber : PieceType;
                              theFromSquare,
                              theToSquare : SquareType); virtual;

{  public}
    { Public declarations }
    constructor Create (aOwner : TComponent); override;
    destructor Destroy; override;

    function GetVersion : TVersionString;

    function WhichSquareIs (MouseX, MouseY : Integer) : SquareType; virtual;

    procedure DrawPieceOnSquare (PieceNumber : PieceType; theSquare : SquareType); virtual;

    procedure Animate; virtual;

    procedure StopAnimation; virtual;

    procedure MakeMove (theChessMove : ChessMoveType); virtual;


    procedure GetChessMove (var theChessMove : ChessMoveType); virtual;

    function GetWhiteIsAtBottom : Boolean;
    procedure SetWhiteIsAtBottom (Yes : Boolean);

    procedure PutPieceOnSquare (thePiece: PieceType;
                                theSquare: SquareType);
    function GetPieceOnSquare(theSquare : SquareType): PieceType;

    procedure RedrawBoard;

  published
    { Published declarations }
    property Version : TVersionString
      read GetVersion;

    property OnMouseDown;

    property WhiteIsAtBottom : Boolean
      read GetWhiteIsAtBottom
      write SetWhiteIsAtBottom;
  end;

{ procedure Register;  }

function SquareIsLight (theSquare : SquareType) : Boolean;

{======================================================================}
implementation


{ procedure Register;
begin
  RegisterComponents('Samples', [TChessDiagram]);
end;   }


constructor TChessDiagram.Create (aOwner : TComponent);
begin
  inherited Create (aOwner);

  fVersion := '1.0';

  fWhiteIsAtBottom := True;

  fItsChessMove.FromSquare     := 0;
  fItsChessMove.ToSquare       := 0;
  fItsChessMove.PromotionPiece := kEmptySquare;
  fItsChessMove.Diacritic      := 1;

  fPromotionPiece := kWhiteQueen;
  fDraggingAPiece := False;
  fAnimatingAMove := False;
  fPieceBeingDragged := kWhitePawn;

  fOriginalSquare := 0;
  fTargetSquare := 0;

  fChessPosition := TChessPosition.Create;
end;


destructor TChessDiagram.Destroy;
begin
  fChessPosition.Free;

  inherited Destroy;
end;


function TChessDiagram.GetVersion : TVersionString;
begin
  GetVersion := fVersion;
end;


function TChessDiagram.GetWhiteIsAtBottom : Boolean;
begin
  GetWhiteIsAtBottom := fWhiteIsAtBottom;
end;


procedure TChessDiagram.SetWhiteIsAtBottom (Yes : Boolean);
begin
  fWhiteIsAtBottom := Yes;
  Paint;
end;


procedure TChessDiagram.StartAnimation (PieceNumber : PieceType;
                                     theFromSquare,
                                     theToSquare : SquareType);
begin
    { Update the board for the source and target squares. }
    { NOTE: Special squares will need extra help! (castling, }
    { en passant) }
  fChessPosition.PutPieceOnSquare (kEmptySquare, theFromSquare);
  fChessPosition.PutPieceOnSquare (PieceNumber, theToSquare);


  fOriginalSquare := theFromSquare;
  fTargetSquare := theToSquare;
end;



procedure TChessDiagram.Animate;
begin
end;



procedure TChessDiagram.StopAnimation;
begin
end;



procedure TChessDiagram.MakeMove (theChessMove : ChessMoveType);
var
  thePiece : PieceType;
  theSquare : SquareType;

begin
  thePiece := fChessPosition.PieceOnSquare (theChessMove.FromSquare);

    { If we're asked to animate a nonexistent piece }
    { then bail. }
  if (thePiece = kEmptySquare)
    then Exit;

  StopAnimation;

  StartAnimation (thePiece,
                  theChessMove.FromSquare,
                  theChessMove.ToSquare);
end;

procedure TChessDiagram.RefreshSquare (theSquare : SquareType);
begin
end;


procedure TChessDiagram.DrawPiece (PieceNumber : PieceType; Hor, Ver : Integer);
begin
end;


procedure TChessDiagram.DrawPieceOnSquare (PieceNumber : PieceType; theSquare : SquareType);
begin
end;


function TChessDiagram.WhichSquareIs (MouseX, MouseY : Integer) : SquareType;
begin
    { This value is illegal and this method must be overriden! }
  WhichSquareIs := -1;
end;


procedure TChessDiagram.ConstrictCoordinates (var X, Y : Integer);
begin
end;


procedure TChessDiagram.GetChessMove (var theChessMove : ChessMoveType);
begin
  theChessMove := fItsChessMove;
end;

 procedure TChessDiagram.RedrawBoard;

 begin

    paint;

 end;

procedure TChessDiagram.PutPieceOnSquare (thePiece: PieceType;
                                       theSquare: SquareType);
begin
  fChessPosition.PutPieceOnSquare (thePiece,
                                   theSquare);

  DrawPieceOnSquare (thePiece,
                     theSquare);
end;

function TChessDiagram.GetPieceOnSquare(theSquare : SquareType): PieceType;
begin
  GetPieceOnSquare := fChessPosition.PieceOnSquare(theSquare);
end;




function SquareIsLight (theSquare : SquareType) : Boolean;
begin
  SquareIsLight := Odd (theSquare div 10 + theSquare - (theSquare div 10) * 10);
end;




end.
